package sun.security.util;

import java.security.SignatureException;
import java.security.spec.ECGenParameterSpec;
import java.security.spec.InvalidParameterSpecException;
import java.security.spec.ECParameterSpec;
import java.security.NoSuchAlgorithmException;
import java.security.AlgorithmParameters;
import java.security.Provider;
import java.io.IOException;
import java.math.BigInteger;
import java.util.Arrays;
import java.security.spec.ECPoint;
import java.security.spec.EllipticCurve;

public final class ECUtil
{
    public static ECPoint decodePoint(byte[] array, EllipticCurve ellipticCurve) throws IOException {
        if (array.length == 0 || array[0] != 4) {
            throw new IOException("Only uncompressed point format supported");
        }
        int n = (array.length - 1) / 2;
        if (n != ellipticCurve.getField().getFieldSize() + 7 >> 3) {
            throw new IOException("Point does not match field size");
        }
        return new ECPoint(new BigInteger(1, Arrays.copyOfRange(array, 1, 1 + n)), new BigInteger(1, Arrays.copyOfRange(array, n + 1, n + 1 + n)));
    }
    
    public static byte[] encodePoint(ECPoint ecPoint, EllipticCurve ellipticCurve) {
        int n = ellipticCurve.getField().getFieldSize() + 7 >> 3;
        byte[] trimZeroes = trimZeroes(ecPoint.getAffineX().toByteArray());
        byte[] trimZeroes2 = trimZeroes(ecPoint.getAffineY().toByteArray());
        if (trimZeroes.length > n || trimZeroes2.length > n) {
            throw new RuntimeException("Point coordinates do not match field size");
        }
        byte[] array = new byte[1 + (n << 1)];
        array[0] = 4;
        System.arraycopy(trimZeroes, 0, array, n - trimZeroes.length + 1, trimZeroes.length);
        System.arraycopy(trimZeroes2, 0, array, array.length - trimZeroes2.length, trimZeroes2.length);
        return array;
    }
    
    public static byte[] trimZeroes(byte[] original) {
        int from;
        for (from = 0; from < original.length - 1 && original[from] == 0; ++from) {}
        if (from == 0) {
            return original;
        }
        return Arrays.copyOfRange(original, from, original.length);
    }
    
    private static AlgorithmParameters getECParameters(Provider provider) {
        try {
            if (provider != null) {
                return AlgorithmParameters.getInstance("EC", provider);
            }
            return AlgorithmParameters.getInstance("EC");
        }
        catch (NoSuchAlgorithmException cause) {
            throw new RuntimeException(cause);
        }
    }
    
    public static byte[] encodeECParameterSpec(Provider provider, ECParameterSpec ecParameterSpec) {
        AlgorithmParameters ecParameters = getECParameters(provider);
        try {
            ecParameters.init(ecParameterSpec);
        }
        catch (InvalidParameterSpecException ex) {
            throw new RuntimeException("Not a known named curve: " + ecParameterSpec);
        }
        try {
            return ecParameters.getEncoded();
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
    }
    
    public static ECParameterSpec getECParameterSpec(Provider provider, ECParameterSpec paramSpec) {
        AlgorithmParameters ecParameters = getECParameters(provider);
        try {
            ecParameters.init(paramSpec);
            return (ECParameterSpec)ecParameters.getParameterSpec(ECParameterSpec.class);
        }
        catch (InvalidParameterSpecException ex) {
            return null;
        }
    }
    
    public static ECParameterSpec getECParameterSpec(Provider provider, byte[] params) throws IOException {
        AlgorithmParameters ecParameters = getECParameters(provider);
        ecParameters.init(params);
        try {
            return (ECParameterSpec)ecParameters.getParameterSpec(ECParameterSpec.class);
        }
        catch (InvalidParameterSpecException ex) {
            return null;
        }
    }
    
    public static ECParameterSpec getECParameterSpec(Provider provider, String stdName) {
        AlgorithmParameters ecParameters = getECParameters(provider);
        try {
            ecParameters.init(new ECGenParameterSpec(stdName));
            return (ECParameterSpec)ecParameters.getParameterSpec(ECParameterSpec.class);
        }
        catch (InvalidParameterSpecException ex) {
            return null;
        }
    }
    
    public static ECParameterSpec getECParameterSpec(Provider provider, int n) {
        AlgorithmParameters ecParameters = getECParameters(provider);
        try {
            ecParameters.init(new ECKeySizeParameterSpec(n));
            return (ECParameterSpec)ecParameters.getParameterSpec(ECParameterSpec.class);
        }
        catch (InvalidParameterSpecException ex) {
            return null;
        }
    }
    
    public static String getCurveName(Provider provider, ECParameterSpec paramSpec) {
        AlgorithmParameters ecParameters = getECParameters(provider);
        ECGenParameterSpec ecGenParameterSpec;
        try {
            ecParameters.init(paramSpec);
            ecGenParameterSpec = (ECGenParameterSpec)ecParameters.getParameterSpec(ECGenParameterSpec.class);
        }
        catch (InvalidParameterSpecException ex) {
            return null;
        }
        if (ecGenParameterSpec == null) {
            return null;
        }
        return ecGenParameterSpec.getName();
    }
    
    public static boolean equals(ECParameterSpec ecParameterSpec, ECParameterSpec ecParameterSpec2) {
        return ecParameterSpec == ecParameterSpec2 || (ecParameterSpec != null && ecParameterSpec2 != null && ecParameterSpec.getCofactor() == ecParameterSpec2.getCofactor() && ecParameterSpec.getOrder().equals(ecParameterSpec2.getOrder()) && ecParameterSpec.getCurve().equals(ecParameterSpec2.getCurve()) && ecParameterSpec.getGenerator().equals(ecParameterSpec2.getGenerator()));
    }
    
    public static byte[] encodeSignature(byte[] array) throws SignatureException {
        try {
            int n = array.length >> 1;
            byte[] array2 = new byte[n];
            System.arraycopy(array, 0, array2, 0, n);
            BigInteger bigInteger = new BigInteger(1, array2);
            System.arraycopy(array, n, array2, 0, n);
            BigInteger bigInteger2 = new BigInteger(1, array2);
            DerOutputStream derOutputStream = new DerOutputStream(array.length + 10);
            derOutputStream.putInteger(bigInteger);
            derOutputStream.putInteger(bigInteger2);
            return new DerValue((byte)48, derOutputStream.toByteArray()).toByteArray();
        }
        catch (Exception cause) {
            throw new SignatureException("Could not encode signature", cause);
        }
    }
    
    public static byte[] decodeSignature(byte[] array) throws SignatureException {
        try {
            DerInputStream derInputStream = new DerInputStream(array, 0, array.length, false);
            DerValue[] sequence = derInputStream.getSequence(2);
            if (sequence.length != 2 || derInputStream.available() != 0) {
                throw new IOException("Invalid encoding for signature");
            }
            BigInteger positiveBigInteger = sequence[0].getPositiveBigInteger();
            BigInteger positiveBigInteger2 = sequence[1].getPositiveBigInteger();
            byte[] trimZeroes = trimZeroes(positiveBigInteger.toByteArray());
            byte[] trimZeroes2 = trimZeroes(positiveBigInteger2.toByteArray());
            int max = Math.max(trimZeroes.length, trimZeroes2.length);
            byte[] array2 = new byte[max << 1];
            System.arraycopy(trimZeroes, 0, array2, max - trimZeroes.length, trimZeroes.length);
            System.arraycopy(trimZeroes2, 0, array2, array2.length - trimZeroes2.length, trimZeroes2.length);
            return array2;
        }
        catch (Exception cause) {
            throw new SignatureException("Invalid encoding for signature", cause);
        }
    }
    
    private ECUtil() {
    }
}
